﻿using System;
using System.ComponentModel;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
namespace FSIInfragistics.Win.UltraWinGrid
{
    public class FsiUltraGrid : UltraGrid
    {
        public FsiUltraGrid() : base()
        {

        }

        private bool convertUtcDate = false;

        [DefaultValue(false)]
        [Category("Appearance")]
        [Description("Treat datetime values as UTC and display as local time")]
        public bool ConvertUtcDate { get { return convertUtcDate; } set { convertUtcDate = value; } }

        #region UtcToLocalConversionDataFilter class
        public class UtcToLocalConversionDataFilter : IEditorDataFilter
        {
            #region IEditorDataFilter Members

            object IEditorDataFilter.Convert(EditorDataFilterConvertArgs conversionArgs)
            {
                ConversionDirection direction = conversionArgs.Direction;
                UltraGridCell cell = conversionArgs.Context as UltraGridCell;
                DateTime value = conversionArgs.IsValid && conversionArgs.Value is DateTime ? (DateTime)conversionArgs.Value : DateTime.MinValue;

                if (cell == null || value == DateTime.MinValue)
                    return conversionArgs.Value;

                switch (direction)
                {
                    case ConversionDirection.EditorToOwner:
                        {
                            //  When the value is going from the editor back to
                            //  the cell, convert it to universal time.
                            //System.Diagnostics.Debug.WriteLine("Converting " + value.ToLongTimeString() + " to " + value.ToUniversalTime().ToLongTimeString());
                            conversionArgs.Handled = true;
                            return value.ToUniversalTime();
                        }
                        break;

                    case ConversionDirection.OwnerToEditor:
                        {
                            //  When the value is going from the cell to
                            //  the editor, convert it to local time.
                            //System.Diagnostics.Debug.WriteLine("Converting " + value.ToLongTimeString() + " to " + value.ToLocalTime().ToLongTimeString());
                            conversionArgs.Handled = true;
                            return value.ToLocalTime();
                        }
                        break;

                }

                return conversionArgs.Value;
            }

            #endregion
        }
        #endregion UtcToLocalConversionDataFilter class

        protected override void OnInitializeLayout(InitializeLayoutEventArgs e)
        {
            base.OnInitializeLayout(e);
           if (! DesignMode)
           {
               if (convertUtcDate)
               {

                   foreach (var x in e.Layout.Bands)
                   {
                       int col = 0;
                       int maxcol = x.Columns.Count;
                       for (col = 0; col < maxcol; col++)
                       {
                           var y = x.Columns[col];
                           // assume all dates are UTC
                           if (y.DataType == typeof(DateTime))
                           {
                               if (y.Editor == null) y.Editor = new DateTimeEditor();
                               y.Editor.DataFilter = new UtcToLocalConversionDataFilter();
                           }
                       }
                   }
               }
            }
        }
    }
}
